@@ -191,7 +191,7 @@ class Job(Entity):
         if os.path.splitext(server_path)[1] == '':
             server_path += '/' + os.path.basename(server_path) + '.zip'
         
-        self.local_path.set(local_path)
+        self.local_path.set(local_path.replace('\\', '/'))
         self.server_path.set(server_path.replace('\\', '/'))
     
     def on_processed(self):
