@@ -1,6 +1,7 @@
 import os
 import json
 import getpass
+import fnmatch
 
 from kabaret import flow
 
@@ -8,7 +9,11 @@ from kabaret import flow
 class MyBookmarks(flow.DynamicMap):
 
     def mapped_names(self, page_num=0, page_size=None):
-        return self.root().project().get_user_bookmarks().mapped_names()
+        user = self.root().project().get_user()
+        if user is not None:
+            return user.bookmarks.mapped_names()
+        else:
+            return []
 
     @classmethod
     def mapped_type(cls):
@@ -19,7 +24,7 @@ class MyBookmarks(flow.DynamicMap):
 
     def _fill_row_cells(self, row, item):
         name = item.name()
-        bookmarks = self.root().project().get_user_bookmarks()
+        bookmarks = self.root().project().get_user().bookmarks
         oid = bookmarks[name].goto_oid.get()
         objects = (
             self.root()
@@ -32,20 +37,51 @@ class MyBookmarks(flow.DynamicMap):
 
     def _fill_row_style(self, style, item, row):
         name = item.name()
-        bookmarks = self.root().project().get_user_bookmarks()
+        bookmarks = self.root().project().get_user().bookmarks
         oid = bookmarks[name].goto.oid()
         style["activate_oid"] = oid
+    
+    def _fill_ui(self, ui):
+        super(MyBookmarks, self)._fill_ui(ui)
+        ui['hidden'] = self.root().project().get_user() is None
 
 
+class GotoPreferences(flow.Action):
+
+    _profile = flow.Parent()
+
+    def allow_context(self, context):
+        user = self.root().project().get_user(
+            self._profile.current_user_id.get()
+        )
+        return (
+            context
+            and context.endswith('.inline')
+            and user is not None
+        )
+
+    def needs_dialog(self):
+        return False
+    
+    def run(self, button):
+        user_pref_oid = '/'.join([
+            self.root().project().admin.users.oid(),
+            self._profile.current_user_id.get(),
+            'preferences'
+        ])
+        return self.get_result(goto=user_pref_oid)
     
 
 class UserProfile(flow.Object):
     '''
     This part is made for the user
     '''
+
+    ICON = ('icons.gui', 'user')
+
     current_user_id = flow.Computed(cached=True)
     my_bookmarks = flow.Child(MyBookmarks).ui(expanded=True)
-
+    preferences = flow.Child(GotoPreferences)
 
     def compute_child_value(self, child_value):
         if child_value is self.current_user_id:
@@ -67,6 +103,15 @@ class UserProfile(flow.Object):
 
             # Return local user name otherwise
             child_value.set(getpass.getuser())
+    
+    def summary(self):
+        user = self.root().project().get_user(self.current_user_id.get())
+        if user is None:
+            return (
+                '<font color=#D5000D>User '
+                f'<b>{self.current_user_id.get()}</b>'
+                ' undefined.</font>'
+            )
 
     def _fill_ui(self, ui):
         if self.root().project().show_login_page():
@@ -90,7 +135,7 @@ class GotoBookmarkAction(flow.Action):
 
     def run(self, button):
         # This is overkill, but needed to the dynamicMap can use the same actions
-        bookmarks = self.root().project().get_user_bookmarks()
+        bookmarks = self.root().project().get_user().bookmarks
         return self.get_result(goto=bookmarks[self._bookmark.name()].goto_oid.get())
 
 
@@ -104,7 +149,7 @@ class RemoveFromBookmark(flow.Action):
 
     def run(self, button):
         # This is overkill, but needed to the dynamicMap can use the same actions
-        bookmarks = self.root().project().get_user_bookmarks()
+        bookmarks = self.root().project().get_user().bookmarks
         bookmarks.remove_bookmark(self._bookmark.name())
         self._bookmarks.touch()
 
@@ -126,7 +171,7 @@ class ToggleBookmarkAction(flow.Action):
         return context and context.endswith(".details")
 
     def get_bookmarks(self):
-        return self.root().project().get_user_bookmarks()
+        return self.root().project().get_user().bookmarks
 
     def is_bookmarked(self):
         return self.get_bookmarks().has_bookmark(self._obj.oid())
@@ -206,6 +251,133 @@ class UserBookmarks(flow.Map):
         style["activate_oid"] = item.goto.oid()
 
 
+class PresetValueManager(flow.Object):
+
+    _value = flow.Parent()
+
+
+class PresetValue(flow.values.Value):
+
+    def get_preset(self):
+        return self._get_presets().get_preset(self.oid())
+    
+    def apply_preset(self):
+        preset = self.get_preset()
+
+        if preset is not None:
+            self.set(preset)
+    
+    def update_preset(self):
+        presets = self._get_presets()
+        presets.update_preset(self.oid(), self.get())
+    
+    def _get_presets(self):
+        user = self.root().project().get_user()
+        return user.preferences.option_presets
+
+
+class PresetChoiceValue(PresetValue):
+
+    def apply_preset(self):
+        preset = self.get_preset()
+
+        if preset is not None and preset in self.choices():
+            self.set(preset)
+
+
+class PresetSessionValue(flow.values.SessionValue):
+
+    def get_preset(self):
+        return self._get_presets().get_preset(self.oid())
+
+    def apply_preset(self):
+        preset = self.get_preset()
+
+        if preset is not None:
+            self.set(preset)
+    
+    def update_preset(self):
+        presets = self._get_presets()
+        presets.update_preset(self.oid(), self.get())
+    
+    def _get_presets(self):
+        user = self.root().project().get_user()
+        return user.preferences.option_presets
+
+
+class OptionPreset(flow.Object):
+
+    contexts = flow.OrderedStringSetParam()
+    context_values = flow.DictParam({})
+    default_value = flow.Param()
+
+
+class BoolOptionPreset(OptionPreset):
+
+    default_value = flow.BoolParam()
+
+
+class OptionPresets(flow.Map):
+
+    @classmethod
+    def mapped_type(cls):
+        return OptionPreset
+    
+    def get_preset(self, param_oid):
+        param_context, param_name = param_oid.rsplit('/', maxsplit=1)
+        value = None
+
+        if self.has_mapped_name(param_name):
+            preset = self.get_mapped(param_name)
+            context_values = preset.context_values.get()
+
+            if not context_values:
+                value = preset.default_value.get()
+            else:
+                for context in preset.contexts.get():
+                    if fnmatch.fnmatch(param_context, context):
+                        return context_values[context]
+                
+                # No contextual preset defined or matching the given param context
+                value = preset.default_value.get()
+        
+        return value
+    
+    def update_preset(self, param_oid, value):
+        param_context, param_name = param_oid.rsplit('/', maxsplit=1)
+
+        if not self.has_mapped_name(param_name):
+            ui = self.root().session().cmds.Flow.get_object_ui(param_oid)
+            
+            if ui.get('editor_type', None) == 'bool':
+                preset = self.add_bool_preset(param_name)
+            else:
+                preset = self.add(param_name)
+        else:
+            preset = self.get_mapped(param_name)
+            context_values = preset.context_values.get()
+
+            if not context_values:
+                preset.default_value.set(value)
+                return
+            else:
+                for context in preset.contexts.get():
+                    if fnmatch.fnmatch(param_context, context):
+                        context_values[context] = value
+                        preset.context_values.set(context_values)
+                        return
+        
+        # No contextual preset defined or matching the given param context
+        preset.default_value.set(value)
+    
+    def add_bool_preset(self, name):
+        return super(OptionPresets, self).add(name, object_type=BoolOptionPreset)
+
+
+class UserPreferences(flow.Object):
+
+    option_presets = flow.Child(OptionPresets)
+
 
 class UserStatus(flow.values.ChoiceValue):
 
@@ -240,25 +412,33 @@ class User(flow.Object):
 
     ICON = ("icons.gui", "user")
 
-    kitsu_id = flow.Param("").ui(label="Kitsu login")
-    kitsu_uid = flow.Param("").ui(label="Kitsu ID")
+    kitsu_id = flow.Param("").ui(label="Kitsu login", editable=False)
+    kitsu_uid = flow.Param("").ui(label="Kitsu ID", editable=False)
     status = flow.Param("User", UserStatus).ui(editable=False)
 
     bookmarks = flow.Child(UserBookmarks)
+    preferences = flow.Child(UserPreferences)
 
 
 class Users(flow.Map):
 
     ICON = ("icons.gui", "team")
+    STYLE_BY_STATUS = {
+        'User': ('icons.gui', 'user'),
+        'Admin': ('icons.gui', 'user-admin'),
+        'Supervisor': ('icons.gui', 'user-lead')
+    }
 
     add_user_action = flow.Child(AddUserAction).ui(label="Add user")
 
+    _name_by_uid = flow.HashParam()
+
     @classmethod
     def mapped_type(cls):
         return flow.injection.injectable(User)
 
     def columns(self):
-        return ["ID", "Kitsu login", "Kitsu ID", "Status"]
+        return ['ID', 'Kitsu login']
 
     def is_admin(self, username):
         user = self.get_mapped(username)
@@ -275,6 +455,8 @@ class Users(flow.Map):
         user.kitsu_uid.set(kitsu_id)
         user.status.set(status)
 
+        self._name_by_uid.set_key(kitsu_id, name)
+
         return user
     
     def get_user_name(self, kitsu_id):
@@ -291,18 +473,20 @@ class Users(flow.Map):
         
         return name
     
-    def get_user(self, kitsu_id):
-        for u in self.mapped_items():
-            if u.kitsu_uid.get() == kitsu_id:
-                return u
-        
-        return None
+    def get_user_by_uid(self, uid):
+        try:
+            return self.get_mapped(
+                self._name_by_uid.get_key(uid)
+            )
+        except KeyError:
+            return None
 
     def _fill_row_cells(self, row, item):
         row["ID"] = item.name()
         row["Kitsu login"] = item.kitsu_id.get()
-        row["Kitsu ID"] = item.kitsu_uid.get()
-        row["Status"] = item.status.get()
+    
+    def _fill_row_style(self, style, item, row):
+        style['icon'] = self.STYLE_BY_STATUS[item.status.get()]
 
 
 class AddEnvVarAction(flow.Action):
