@@ -6,6 +6,36 @@ import libreflow.utils.kabaret as kutils
 from .. import runner_factory
 
 
+@SubprocessManagerCmds.cmd
+class Run(Cmd):
+    def _decode(
+        self,
+        runner_name,
+        tags=[],
+        version=None,
+        label=None,
+        extra_argv=[],
+        extra_env={},
+    ):
+        self.runner_name = runner_name
+        self.tags = tags
+        self.version = version
+        self.label = label
+        self.extra_argv = extra_argv
+        self.extra_env = extra_env
+
+    def _execute(self):
+        rid = self.actor().run(
+            self.runner_name,
+            self.tags,
+            self.version,
+            self.label,
+            self.extra_argv,
+            self.extra_env,
+        )
+        return rid
+
+
 @SubprocessManagerCmds.cmd
 class List_Runner_Infos(Cmd):
     """
@@ -135,6 +165,21 @@ class SubprocessManager(ksm.actor.SubprocessManager):
         
         return deleted
     
+    def run(
+        self,
+        runner_name,
+        tags=[],
+        version=None,
+        label=None,
+        extra_argv=[],
+        extra_env={},
+    ):
+        runner = self._factories.get_runner(
+            runner_name, tags, version, label, extra_argv, extra_env,
+        )
+        rid = self._subprocess_manager.run(runner)
+        return rid
+
     # def get_runner_from_pid(self, pid):
     #     """
     #     Returns the runner which launched the process
